//	Copyright 1996 Be Inc., All Rights Reserved.
//  Modified by T. Yamagata for Preview Release Version of BeOS
//	Update: 971225/14:05
//	Last update: 981202 

#include "StringLabel.h"

#include <View.h>
#include <string.h>

StringLabel::StringLabel(const char *aString, const BPoint origin, 
	const float fontSize )
{
	fString = new char[strlen(aString)+1];
	strcpy(fString,aString);
	fOrigin = origin;
	fAlignment = B_ALIGN_LEFT;
	fSize = fontSize;
	fColor.red = 0;
	fColor.green = 0;
	fColor.blue = 0;
	fColor.alpha = 0;
	
	Invalidate();
}

//============================================================
// Method: Destructor
// 
// The usual cleanup destructor.  Delete the memory for the 
// string that we copied in.
//============================================================
StringLabel::~StringLabel()
{
	delete [] fString;
}

//============================================================
// Method: Draw
//
// This is the meat and potatoes method.  It is completely
// responsible for drawing the string into the view according
// to the various flags that have been set.
//
// In this method we first check to see whether things need to
// be recalculated because one of the parameters changed.
// After that, we set the text parameters, move into position,
// and draw.
//============================================================

void
StringLabel::Draw(BView* aView)
{
	if (fNeedsCalculation)
		Recalculate(aView);
		
	// Setting font info
	BFont	font;
	aView->GetFont(&font);

	font.SetSize(fSize);
	aView->SetFont(&font);
	
	aView->SetHighColor(fColor);
	aView->MovePenTo(fStartPoint);
	aView->SetDrawingMode(B_OP_OVER);
	aView->DrawString(fString);
}


//============================================================
// Method: Invalidate()
//
// Invalidate simply informs the label that before it draws
// the next time, it must re-calculate its origin.  We do it
// this way instead of calling the Recalculate method directly
// becuase the sizing information is dependent on what view
// we are drawing into.  We don't know that until we actually
// draw, so that is the proper place to do the calculation.
//============================================================

void
StringLabel::Invalidate()
{
	fNeedsCalculation = 1;
}

//============================================================
// Method: Recalculate
//
// Here is where we re-calculate the drawing origin based on
// the alignment, and size of the text string.  The various
// text parameters are set on the view, then the StringWidth
// method is called.  Using this information we then adjust
// the fStartPoint variable accordingly.
//============================================================

void
StringLabel::Recalculate(BView* aView)
{
	// Start by assuming left alignment	
	fStartPoint = fOrigin;
	BFont 	font;
		// Setting font info so we can do calculations	
		aView->GetFont(&font);
		font.SetSize(fSize);								//981202
		aView->SetFont(&font);	
	long aLength = 0;
	
	switch (fAlignment)
	{
		case B_ALIGN_LEFT:
		break;
		
		case B_ALIGN_CENTER:
			aLength = (long)aView->StringWidth(fString);	//981030
			aLength = aLength / 2;
		break;
		
		case B_ALIGN_RIGHT:
			aLength = (long)aView->StringWidth(fString);	//981030
		break;
	}
	
	fStartPoint.x = fOrigin.x - aLength;
	fNeedsCalculation = 0;
}

//============================================================
// Method: SetText
//
// Set the text to be displayed by the label.  We make a copy
// of the null terminated string that is passed in and then
// invalidate the label.
//============================================================

void
StringLabel::SetText(const char *aString)
{
	delete fString;
	fString = new char[strlen(aString)+1];
	strcpy(fString,aString);
	
	Invalidate();
}

//============================================================
// Method: SetAlignment
//
// Alignment can be:
// ALIGN_LEFT, ALIGN_CENTER, ALIGN_RIGHT
// The alignment determines how the text is displayed relative
// to the origin that is set.
//						Origin
//							|
// ALIGN_LEFT:  			Left Aligned Label
// ALIGN_CENTER:      Center Aligned
// ALIGN_RIGHT: Right Aligned
//============================================================

void
StringLabel::SetAlignment(const alignment theAlignment)
{
	fAlignment = theAlignment;
	
	Invalidate();
}

//============================================================
// Method: SetOrigin
// 
// The origin is the starting point for where the text
// will be displayed.  Depending on the type of alignment,
// The origin will either be at the left, center, or right
// of the text.
//============================================================
 
void
StringLabel::SetOrigin(const BPoint& origin)
{
	fOrigin = origin;

	Invalidate();
}

void
StringLabel::SetSize(const float fontSize)
{
	fSize = fontSize;

	Invalidate();
}

//============================================================
//	Method: SetColor
//
// Sets the color of the label.  This allows you to have
// labels of any color.
//============================================================

void
StringLabel::SetColor(rgb_color aColor)
{
	fColor = aColor;
}


